from typing import MutableMapping, Optional

import eventum_plugins.input.time_patterns as models
import streamlit as st
from eventum_content_manager.manage import (ContentManagementError,
                                            get_time_pattern_filenames,
                                            load_time_pattern)
from pydantic import ValidationError

from eventum_studio.components.component import (BaseComponent,
                                                 ComponentActionError)
from eventum_studio.components.time_pattern_configurator import \
    TimePatternConfigurator
from eventum_studio.notifiers import NotificationLevel, default_notifier
from eventum_studio.utils.validation_prettier import prettify_errors
from eventum_studio.widget_management import WidgetKeysContext


class TimePatternConfiguratorList(BaseComponent):
    """Component for managing list of configurators."""

    _TIME_PATTERN_COLORS = ('blue', 'green', 'orange', 'red', 'violet')
    _MAX_LIST_SIZE = 5

    def __init__(
        self,
        session_state: MutableMapping = st.session_state,
        id: int = 1,
        widget_keys_context: Optional[WidgetKeysContext] = None,
        props: Optional[dict] = None
    ) -> None:
        self._check_time_pattern_colors()
        super().__init__(session_state, id, widget_keys_context, props)

    def _init_state(self):
        self._session_state['time_pattern_id_counter'] = 1
        self._session_state['time_patterns_counter'] = 0
        self._session_state['time_pattern_ids'] = []
        self._session_state['available_colors'] = set(
            TimePatternConfiguratorList._TIME_PATTERN_COLORS
        )
        self._session_state['given_colors'] = dict()
        self._session_state['loaded_time_pattern_filenames'] = set()

    def _show_manage_buttons(self):
        is_max_len = len(
            self._session_state['time_pattern_ids']
        ) >= self._MAX_LIST_SIZE

        st.button(
            'Create new',
            key=self._wk.get_ephemeral(),
            disabled=is_max_len,
            on_click=lambda: self._add(),
            use_container_width=True
        )
        col1, col2 = st.columns([7, 3])

        selected_pattern = col1.selectbox(
            'Time patterns',
            options=get_time_pattern_filenames(),
            key=self._wk('pattern_selected_for_load'),
            label_visibility='collapsed',
            disabled=is_max_len
        )

        col2.button(
            'Load',
            key=self._wk.get_ephemeral(),
            disabled=(
                is_max_len
                or not selected_pattern
            ),
            on_click=lambda: self._load(
                filename=selected_pattern
            ),
            use_container_width=True,
        )

        if is_max_len:
            st.write('*:grey[Maximum number of patterns]*')

    def _show(self):
        st.title('Time Patterns')
        for id in self._session_state['time_pattern_ids']:
            TimePatternConfigurator(
                id=id,
                widget_keys_context=self._wk,
                props={
                    'save_callback': (
                        lambda filename: self._session_state[
                            'loaded_time_pattern_filenames'
                        ].add(filename)
                    ),
                    'delete_callback': (
                        lambda id=id: self._delete(id)
                    )
                }
            )._show()
        if not self._session_state['time_pattern_ids']:
            st.markdown(
                (
                    '<div style="text-align: center; color: grey;">'
                    'No time patterns. Create or load one.'
                    '</div>'
                ),
                unsafe_allow_html=True
            )
        st.divider()
        self._show_manage_buttons()

    def _add(
        self,
        initial_state: Optional[models.TimePatternConfig] = None,
        pattern_filename: Optional[str] = None
    ) -> None:
        """Add time pattern adjuster element to list."""
        if self._session_state['time_patterns_counter'] == self._MAX_LIST_SIZE:
            raise ComponentActionError(
                f'Max size ({self._MAX_LIST_SIZE}) of list is exceeded'
            )

        id = self._session_state['time_pattern_id_counter']
        self._session_state['time_pattern_ids'].append(id)
        self._session_state['time_pattern_id_counter'] += 1
        self._session_state['time_patterns_counter'] += 1

        color = self._session_state['available_colors'].pop()
        self._session_state['given_colors'][id] = color

        TimePatternConfigurator(
            id=id,
            widget_keys_context=self._wk,
            props={
                'initial_state': initial_state,
                'pattern_filename': pattern_filename,
                'color': color
            }
        )

    def _load(self, filename: str) -> None:
        """Load selected time pattern from repository"""
        if filename in self._session_state['loaded_time_pattern_filenames']:
            default_notifier(
                'Time pattern is already loaded',
                NotificationLevel.WARNING
            )
            return

        try:
            time_pattern_data = load_time_pattern(filename)
            time_pattern = models.TimePatternConfig.model_validate(
                obj=time_pattern_data
            )
        except ContentManagementError as e:
            default_notifier(
                message=f'Failed to load time pattern: {e}',
                level=NotificationLevel.ERROR
            )
            return
        except ValidationError as e:
            default_notifier(
                message=prettify_errors(e.errors()),
                level=NotificationLevel.ERROR
            )
            return

        self._add(
            initial_state=time_pattern,
            pattern_filename=self._session_state['pattern_selected_for_load']
        )
        self._session_state['loaded_time_pattern_filenames'].add(filename)

    def _delete(self, id: int) -> None:
        """Delete specified time pattern adjuster from list."""
        time_pattern = TimePatternConfigurator(
            id=id,
            widget_keys_context=self._wk
        )
        if time_pattern.is_saved():
            self._session_state['loaded_time_pattern_filenames'].remove(
                time_pattern.get_saved_filename()
            )
        time_pattern.release_state()

        self._session_state['time_pattern_ids'].remove(id)
        self._session_state['time_patterns_counter'] -= 1
        self._session_state['available_colors'].add(
            self._session_state['given_colors'].pop(id)
        )

    def get_pattern_configs(self) -> list[models.TimePatternConfig]:
        """Get list of currently adjusted time pattern configs."""
        configs = []
        for id in self._session_state['time_pattern_ids']:
            pattern = TimePatternConfigurator(
                id=id,
                widget_keys_context=self._wk
            )
            try:
                configs.append(pattern.get_current_configuration())
            except ValidationError as e:
                default_notifier(
                    message=(
                        'Failed to validate parameters in time pattern '
                        f'"{pattern.label}: {prettify_errors(e.errors())}'
                    ),
                    level=NotificationLevel.ERROR
                )

        return configs

    def get_pattern_colors(
        self,
        hex_format: bool = True
    ) -> list[str]:
        """Get list of current time pattern colors."""
        colors: dict[int, str] = self._session_state['given_colors']
        if hex_format:
            st_colors = {
                'blue': '#60b4ff',
                'green': '#3dd56d',
                'orange': '#ffbd45',
                'red': '#ff4b4b',
                'violet': '#b27eff'
            }
            return [st_colors[color] for color in colors.values()]
        else:
            return [color for color in colors.values()]

    @classmethod
    def _check_time_pattern_colors(cls) -> None:
        """Check if number of unique time pattern colors is greater or equal
        to `_MAX_LIST_SIZE`. Raise ValueError otherwise.
        """
        unique_colors = len(set(cls._TIME_PATTERN_COLORS))
        if unique_colors < cls._MAX_LIST_SIZE:
            raise ValueError(
                f'`_MAX_SIZE` is set to {cls._MAX_LIST_SIZE} but there are '
                f'only {unique_colors} unique colors in `_LABEL_COLORS`'
            )



import streamlit as st
from eventum_content_manager.manage import (ContentManagementError,
                                            get_csv_sample_filenames,
                                            load_csv_sample)

from eventum_studio.components.component import BaseComponent
from eventum_studio.notifiers import NotificationLevel, default_notifier


class SampleExplorer(BaseComponent):
    """Component for displaying sample content."""

    _SHOW_PROPS = {
        'display_size': int
    }

    def _show(self) -> None:
        st.caption('Sample explorer')
        sample = st.selectbox(
            'Sample',
            options=get_csv_sample_filenames(),
            help='Select sample from repository to preview it in below table'
        )
        try:
            if sample is not None:
                sample_data = load_csv_sample(  # type: ignore[assignment]
                    path=sample
                )
            else:
                sample_data = []                # type: ignore[assignment]
        except ContentManagementError as e:
            default_notifier(
                message=f'Failed to load sample: {e}',
                level=NotificationLevel.ERROR
            )
            sample_data = []                    # type: ignore[assignment]

        total_size = len(sample_data)
        display_size = self._props['display_size']

        st.table(sample_data[:display_size])

        if total_size > display_size:
            st.text(f'and {total_size - display_size} more ...')



from typing import Any, Callable, Optional

import streamlit as st
from eventum_content_manager.manage import (ContentManagementError,
                                            get_template_filenames,
                                            load_template, save_template)

from eventum_studio.components.component import (BaseComponent,
                                                 ComponentActionError)
from eventum_studio.notifiers import NotificationLevel, default_notifier


class TemplateManager(BaseComponent):
    """Component for managing templates."""

    _SHOW_PROPS = {
        'get_content_callback': Callable[[], str],
        'set_content_callback': Callable[[str], Any]
    }

    def _init_state(self):
        self._session_state['is_empty'] = True
        self._session_state['template_filename'] = ''
        self._session_state['is_saved'] = False

    def _show_manage_buttons(self):
        is_empty = self._session_state['is_empty']

        st.button(
            'Create new',
            key=self._wk.get_ephemeral(),
            disabled=not is_empty,
            on_click=self._add,
            use_container_width=True
        )
        col1, col2 = st.columns([7, 3])

        selected_template = col1.selectbox(
            'Template',
            options=get_template_filenames(),
            key=self._wk('template_selected_for_load'),
            label_visibility='collapsed',
            disabled=not is_empty
        )

        col2.button(
            'Load',
            key=self._wk.get_ephemeral(),
            disabled=(not is_empty or not selected_template),
            on_click=lambda: self._load(filename=selected_template),
            use_container_width=True,
        )

        if not is_empty:
            st.write('*:grey[Template is added]*')

    def _show_manage_section(self) -> None:
        st.text_input(
            'File name',
            key=self._wk('template_filename'),
            disabled=self._session_state['is_saved'],
            help='Path for current template'
        )

        if self._session_state['is_saved']:
            st.button(
                'Update',
                key=self._wk.get_ephemeral(),
                on_click=lambda: self._save(overwrite=True),
                use_container_width=True,
                type='primary'
            )
        else:
            st.button(
                'Save',
                key=self._wk.get_ephemeral(),
                on_click=self._save,
                use_container_width=True,
                type='primary'
            )
        st.button(
            'Delete',
            key=self._wk.get_ephemeral(),
            on_click=self._clear,
            use_container_width=True,
        )

    def _show(self):
        st.title('Template')
        if self._session_state['is_empty']:
            st.markdown(
                (
                    '<div style="text-align: center; color: grey;">'
                    'No template. Create or load one.'
                    '</div>'
                ),
                unsafe_allow_html=True
            )
        else:
            template_filename = self._session_state['template_filename']
            label = template_filename if template_filename else 'New template'

            with st.expander(label, expanded=True):
                self._show_manage_section()

        st.divider()
        self._show_manage_buttons()

    def _add(
        self,
        initial_state: Optional[str] = None,
        template_filename: Optional[str] = None
    ) -> None:
        """Add new template to configurator."""
        if not self._session_state['is_empty']:
            raise ComponentActionError('Template is already added')

        if initial_state is None:
            initial_state = ''

        try:
            self._props['set_content_callback'](initial_state)
        except Exception as e:
            default_notifier(
                f'Failed to present content of template: {e}',
                NotificationLevel.ERROR
            )
            return

        if template_filename is not None:
            self._session_state['is_saved'] = True
            self._session_state['template_filename'] = template_filename

        self._session_state['is_empty'] = False

    def _load(self, filename: str) -> None:
        """Load selected template from repository to configurator."""
        try:
            template_content = load_template(filename)
        except ContentManagementError as e:
            default_notifier(str(e), NotificationLevel.ERROR)
            return

        self._add(
            initial_state=template_content,
            template_filename=filename
        )

    def _clear(self) -> None:
        """Clear template configurator."""
        self._session_state['is_empty'] = True
        self._session_state['is_saved'] = False
        self._session_state['template_filename'] = ''
        self._props['set_content_callback']('')

    def _save(self, overwrite: bool = False) -> None:
        """Save current template to repository."""
        try:
            content = self._props['get_content_callback']()
        except Exception as e:
            default_notifier(
                f'Failed to get template content: {e}',
                NotificationLevel.ERROR
            )
            return

        try:
            save_template(
                content=content,
                path=self._session_state['template_filename'],
                overwrite=overwrite
            )
        except ContentManagementError as e:
            default_notifier(f'Failed to save: {e}', NotificationLevel.ERROR)
            return

        self._session_state['is_saved'] = True
        default_notifier('Saved in repository', NotificationLevel.SUCCESS)

    @property
    def is_empty(self) -> bool:
        return self._session_state['is_empty']



from typing import Callable

import streamlit as st
from streamlit_elements import editor  # type: ignore[import-untyped]
from streamlit_elements import elements, event, lazy

from eventum_studio.components.component import BaseComponent
from eventum_studio.notifiers import NotificationLevel, default_notifier


class TemplateEditor(BaseComponent):
    """Component for editing template content."""

    _SHOW_PROPS = {
        'content': str,
        'read_only': bool,
        'on_change': Callable[[str], None]
    }

    def _show(self) -> None:
        st.caption(
            'Template content',
            help=(
                'Edit template content you want to render '
                'and use `Ctrl+S` to commit changes'
            )
        )
        with elements(self._wk('template_editor')):
            event.Hotkey(
                sequence='ctrl+s',
                callback=(
                    lambda:
                    default_notifier(
                        message='Template is updated',
                        level=NotificationLevel.INFO
                    )
                ),
                bindInputs=True,
                overrideDefault=True
            )

            editor.Monaco(
                theme='vs-dark',
                language='javascript',
                value=self._props['content'],
                onChange=lazy(self._props['on_change']),
                options={
                    'readOnly': self._props['read_only'],
                    'cursorSmoothCaretAnimation': True
                },
                height=670,
            )



import os
import sys

import streamlit.web.cli as st_cli

BASE_PATH = os.path.dirname(os.path.abspath(__file__))
STUDIO_ENTRYPOINT = os.path.join(BASE_PATH, '01_Time_distribution.py')

THEME_OPTIONS = [
    '--theme.primaryColor=#8282ef',
    '--theme.backgroundColor=#181818',
    '--theme.secondaryBackgroundColor=#252526',
    '--theme.textColor=#e3e3e3',
]


def main() -> None:
    """Execute command to run streamlit app."""
    os.chdir(BASE_PATH)
    sys.argv = ['streamlit', 'run', STUDIO_ENTRYPOINT, *THEME_OPTIONS]
    exit(st_cli.main())


if __name__ == '__main__':
    main()




from abc import ABC, abstractmethod
from copy import deepcopy
from typing import Any, MutableMapping, Optional

import streamlit as st

from eventum_studio.widget_management import (EPHEMERAL_PREFIX,
                                              ContextualSessionState,
                                              WidgetKeysContext)


def persist_state():
    """Persist state after changing displaying page."""
    for k, v in st.session_state.items():
        if k.startswith(EPHEMERAL_PREFIX):
            del st.session_state[k]
        else:
            st.session_state[k] = v


class ComponentPropsError(Exception):
    """Exception indicating that the properties passed to the component
    do not correspond to the expected set of elements.
    """


class ComponentActionError(Exception):
    """Exception for indicating that action performed on component is
    incorrect.
    """


class BaseComponent(ABC):
    """Base class for creating session isolated components."""
    _STATE_INITIALIZATION_PROPS: dict[str, Any] = {}
    _SHOW_PROPS: dict[str, Any] = {}

    def __init__(
        self,
        session_state: MutableMapping = st.session_state,
        id: int = 1,
        widget_keys_context: Optional[WidgetKeysContext] = None,
        props: Optional[dict] = None
    ) -> None:
        self._wk = (
            deepcopy(widget_keys_context)
            if widget_keys_context else WidgetKeysContext()
        )
        self._props = props or dict()

        self._wk.register_component(self.__class__.__name__, id)
        self._session_state = ContextualSessionState(session_state, self._wk)

        self.__init_state_wrapper()

    def __init_state_wrapper(self) -> None:
        """Check whether the session is initialized and call
        initialization in case it's not.
        """
        if 'initialized' in self._session_state:
            return

        expected_keys = self._STATE_INITIALIZATION_PROPS.keys()
        provided_keys = self._props.keys()
        if not set(expected_keys).issubset(provided_keys):
            raise ComponentPropsError(
                f'Expected {list(expected_keys)} props to init component '
                f'session but provided props are {list(provided_keys)}'
            )

        self._init_state()
        self._session_state['initialized'] = True

    def _init_state(self) -> None:
        """Perform state initialization."""
        ...

    def release_state(self) -> None:
        """Delete items from session state added on initialization or used
        as widget keys in `_show` method."""
        self._session_state.delete_context_elements()

    def show(self) -> None:
        """Present component structure."""
        expected_keys = self._SHOW_PROPS.keys()
        provided_keys = self._props.keys()
        if not set(expected_keys).issubset(provided_keys):
            raise ComponentPropsError(
                f'Expected {list(expected_keys)} props to show component '
                f'but provided props are {list(provided_keys)}'
            )

        self._show()

    @abstractmethod
    def _show(self) -> None:
        """Present component structure."""
        ...



from typing import Any, MutableMapping
from uuid import uuid4

EPHEMERAL_PREFIX = '~'


class WidgetKeysContext():
    """Helper class for creating widget keys."""

    _SEPARATOR = ':'
    _ID_PREFIX = '-'

    def __init__(
        self,
    ) -> None:
        self._component_stack: list[str] = []

    def __call__(
        self,
        widget_key: str
    ) -> str:
        return self._SEPARATOR.join(self._component_stack + [widget_key])

    def __contains__(self, __key: str) -> bool:
        """Check if widget key belongs to context."""
        *_, id = __key.rsplit(self._SEPARATOR, maxsplit=1)

        if not id:
            return False

        return self.__call__(id) == __key

    def register_component(self, component_name: str, component_id: int):
        """Add component to context stack."""
        self._component_stack.append(
            f'{component_name}{self._ID_PREFIX}{component_id}'
        )

    @staticmethod
    def get_ephemeral() -> str:
        """Get globally unique ephemeral key. Used for immutable
        widgets such as button."""
        return EPHEMERAL_PREFIX + str(uuid4())


class ContextualSessionState:
    """Wrapper class for streamlit session state that provides isolation
    for widget keys from different contexts.
    """
    def __init__(
        self,
        st_session_state: MutableMapping,
        widget_keys_context: WidgetKeysContext
    ) -> None:
        self._session_state = st_session_state
        self._wk = widget_keys_context

    def __getitem__(self, __key: Any) -> Any:
        return self._session_state[self._wk(__key)]

    def __setitem__(self, __key: Any, __value: Any) -> None:
        self._session_state[self._wk(__key)] = __value

    def __delitem__(self, __key: Any) -> None:
        del self._session_state[self._wk(__key)]

    def __contains__(self, __key: str) -> bool:
        return self._wk(__key) in self._session_state

    def delete_context_elements(self) -> None:
        for key in self._session_state.keys():
            if key in self._wk:
                del self._session_state[key]



import os

import streamlit as st

from eventum_studio.components.component import persist_state
from eventum_studio.components.template_configuration_editor import \
    TemplateConfigurationEditor
from eventum_studio.components.template_editor import TemplateEditor
from eventum_studio.components.template_manager import TemplateManager
from eventum_studio.components.template_renderer import TemplateRenderer
from eventum_studio.components.template_state_viewer import TemplateStateViewer
from eventum_studio.theme import apply_theme

BASE_DIR = os.path.abspath(os.path.dirname(__file__))
ICON_PATH = os.path.join(BASE_DIR, '../static/favicon.ico')

persist_state()
st.set_page_config(
    page_title='Eventum Studio',
    page_icon=ICON_PATH,
    layout='wide',
    initial_sidebar_state='expanded'
)
apply_theme()

for key in ['template_content', 'config_content']:
    if key not in st.session_state:
        st.session_state[key] = ''

manager = TemplateManager(
    props={
        'get_content_callback': lambda: st.session_state['template_content'],
        'set_content_callback': (
            lambda content:
            st.session_state.__setitem__('template_content', content)
        )
    }
)
editor = TemplateEditor(
    props={
        'content': st.session_state['template_content'],
        'read_only': manager.is_empty,
        'on_change': (
            lambda value:
            st.session_state.__setitem__('template_content', value)
        )
    }
)
config_editor = TemplateConfigurationEditor(
    props={
        'on_change': (
            lambda value:
            st.session_state.__setitem__('config_content', value)
        )
    }
)
renderer = TemplateRenderer(
    props={
        'template_content': st.session_state['template_content'],
        'configuration_content': config_editor.content
    }
)
state_viewer = TemplateStateViewer(
    props={
        'local_vars': renderer.local_vars_state,
        'shared_vars': renderer.shared_vars_state,
        'subprocess_commands_history': renderer.subprocess_commands_history,
        'clear_state_callback': renderer.clear_state
    }
)

with st.sidebar:
    manager.show()

editor_tab, configuration_tab, rendering_tab, state_tab = st.tabs(
    ['Template', 'Configuration', 'Rendering', 'State']
)

with editor_tab:
    editor.show()

with configuration_tab:
    config_editor.show()

with rendering_tab:
    renderer.show()

with state_tab:
    state_viewer.show()



import streamlit as st

from eventum_studio.components.component import BaseComponent
from eventum_studio.notifiers import NotificationLevel, default_notifier
from eventum_studio.utils.relative_time import validate_time_span


class SpanInput(BaseComponent):
    """Span input component."""

    def _init_state(self) -> None:
        self._session_state['span_expression'] = '10m'
        self._session_state['previous_span_expression'] = '10m'
        self._session_state['mode'] = 'Auto'

    def _show(self) -> None:
        col1, col2 = st.columns([1, 1])

        col1.radio(
            label='Time span',
            help=(
                'Time span of one bin in histogram.'
            ),
            key=self._wk('mode'),
            options=['Auto', 'Custom'],
            horizontal=True
        )

        col2.text_input(
            'Custom span',
            placeholder='expression',
            help='Expression examples: `1s`, `30m`, `12h`, `7d`, ... etc.',
            key=self._wk('span_expression'),
            disabled=self.is_auto(),
            on_change=self._check_expression
        )

        ss = self._session_state
        ss['previous_span_expression'] = ss['span_expression']

    def _check_expression(self) -> None:
        ss = self._session_state
        if not validate_time_span(ss['span_expression']):
            ss['span_expression'] = ss['previous_span_expression']
            default_notifier(
                message='Incorrect span expression',
                level=NotificationLevel.WARNING
            )

    def is_auto(self) -> bool:
        """Get status whether `Auto` mode is selected."""
        return True if self._session_state['mode'] == 'Auto' else False

    def get_expression(self) -> str:
        """Get current span expression."""
        return self._session_state['span_expression']



import streamlit as st


def apply_theme():
    """Apply theme for page by setting css styles."""
    st.markdown(
        '<style>'
        'code {color: #8282ef}'
        'div[data-testid=stToast] {background-color: #252526}'
        '</style>',
        unsafe_allow_html=True
    )



from datetime import datetime

import streamlit as st
import yaml
from eventum_plugins.event.base import (EventPluginConfigurationError,
                                        EventPluginRuntimeError)
from eventum_plugins.event.jinja import (JinjaEventConfig, JinjaEventPlugin,
                                         State, SubprocessManager,
                                         SubprocessManagerMock, TemplateConfig,
                                         TemplatePickingMode)
from jinja2 import DictLoader
from pydantic import ValidationError
from streamlit_elements import editor, elements  # type: ignore[import-untyped]

from eventum_studio.components.component import BaseComponent
from eventum_studio.notifiers import NotificationLevel, default_notifier
from eventum_studio.utils.validation_prettier import prettify_errors


class TemplateRenderer(BaseComponent):
    """Component for rendering templates."""

    _SHOW_PROPS = {
        'template_content': str,
        'configuration_content': str,
    }

    def _init_state(self) -> None:
        self._session_state['rendering_result'] = ''
        self._session_state['local_vars_state'] = None
        self._session_state['shared_vars_state'] = None
        self._session_state['subprocess_manager'] = SubprocessManagerMock()

        self._session_state['mock_checkbox'] = True

    def _render(self) -> None:
        """Render currently set template content."""

        content = self._props['configuration_content']

        try:
            config_data = yaml.load(content, yaml.Loader)
        except yaml.YAMLError as e:
            default_notifier(
                message=(
                    'Failed to render template due to configuration '
                    f'parse failure: {e}'
                ),
                level=NotificationLevel.ERROR
            )
            return

        if not isinstance(config_data, dict):
            default_notifier(
                message=(
                    'Failed to render template due to invalid configuration: '
                    f'Key-value mapping expected, but got {type(config_data)}'
                ),
                level=NotificationLevel.ERROR
            )
            return

        try:
            config = JinjaEventConfig(
                mode=TemplatePickingMode.ALL,
                templates={
                    'template': TemplateConfig(     # type: ignore[call-arg]
                        template='template.jinja'
                    )
                },
                **config_data
            )
        except ValidationError as e:
            default_notifier(
                message=(
                    'Failed to render template due to invalid configuration: '
                    f'{prettify_errors(e.errors())}'
                ),
                level=NotificationLevel.ERROR
            )
            return

        timestamp = datetime.now().astimezone()
        tz = timestamp.strftime('%z')

        params = {
            'timestamp': timestamp.replace(tzinfo=None).isoformat(),
            'tz': tz
        }

        local_vars: dict | None = self._session_state['local_vars_state']
        shared_vars: State | None = self._session_state['shared_vars_state']
        subprocess_manager = self._session_state['subprocess_manager']

        try:
            plugin = JinjaEventPlugin(
                config=config,
                loader=DictLoader(
                    {'template.jinja': self._props['template_content']}
                )
            )

            if local_vars:
                _, state = local_vars.popitem()
                plugin.local_vars = {'template.jinja': state}

            if shared_vars:
                plugin.shared_vars = shared_vars

            if subprocess_manager:
                plugin.subprocess_manager = subprocess_manager

            result = plugin.render(**params)
        except (EventPluginConfigurationError, EventPluginRuntimeError) as e:
            default_notifier(
                message=(f'Failed to render template: {e}'),
                level=NotificationLevel.ERROR
            )
            return

        self._session_state['rendering_result'] = result.pop()

        self._session_state['local_vars_state'] = plugin.local_vars
        self._session_state['shared_vars_state'] = plugin.shared_vars
        self._session_state['subprocess_manager'] = plugin.subprocess_manager

        default_notifier(
            message=('Rendered successfully'),
            level=NotificationLevel.SUCCESS
        )

    def _show(self) -> None:
        st.caption(
            'Template rendering',
            help='Press render button and see the result in right side'
        )

        with elements(self._wk('template_renderer')):
            editor.MonacoDiff(
                theme='vs-dark',
                language='javascript',
                original=self._props['template_content'],
                modified=self._session_state['rendering_result'],
                options={
                    'readOnly': True,
                    'cursorSmoothCaretAnimation': True
                },
                height=560,
            )

        col1, col2 = st.columns([3, 1])
        col2.button(
            'Render',
            use_container_width=True,
            type='primary',
            on_click=self._render
        )
        col1.checkbox(
            'Mock subprocesses',
            key=self._wk('mock_checkbox'),
            on_change=(
                lambda:
                self._session_state.__setitem__(
                    'subprocess_manager',
                    SubprocessManager()
                    if not self._session_state['mock_checkbox']
                    else SubprocessManagerMock()
                )
            ),
            help='Mock performing subprocesses in template'
        )

    def clear_state(self) -> None:
        """Clear state of locals, shared and history of subprocess
        commands.
        """
        self._session_state['local_vars_state'] = None
        self._session_state['shared_vars_state'] = None
        self._session_state['subprocess_manager'] = (
            SubprocessManager()
            if not self._session_state['mock_checkbox']
            else SubprocessManagerMock()
        )

    @property
    def local_vars_state(self) -> dict:
        """Get state of template local variables."""
        locals: dict[str, State] = self._session_state['local_vars_state']

        if locals is None:
            return {}

        if locals:
            return next(iter(locals.values())).as_dict()
        else:
            return {}

    @property
    def shared_vars_state(self) -> dict:
        """Get state of template shared variables."""
        shared: State = self._session_state['shared_vars_state']

        if shared is None:
            return {}

        return shared.as_dict()

    @property
    def subprocess_commands_history(self) -> tuple[tuple[int, str], ...]:
        """Get history of commands running in templates via `subprocess`."""
        subprocess_manager = self._session_state['subprocess_manager']
        return subprocess_manager.commands_history



import os
from datetime import datetime

import streamlit as st

from eventum_studio.components.component import persist_state
from eventum_studio.components.span_input import SpanInput
from eventum_studio.components.time_pattern_configurator_list import \
    TimePatternConfiguratorList
from eventum_studio.components.time_pattern_distribution_histogram import \
    TimePatternDistributionHistogram
from eventum_studio.theme import apply_theme

BASE_DIR = os.path.abspath(os.path.dirname(__file__))
ICON_PATH = os.path.join(BASE_DIR, './static/favicon.ico')

persist_state()
st.set_page_config(
    page_title='Eventum Studio',
    page_icon=ICON_PATH,
    layout='wide',
    initial_sidebar_state='expanded'
)
apply_theme()

configs_list = TimePatternConfiguratorList()
with st.sidebar:
    configs_list.show()

col1, col2 = st.columns([1, 1])


with col1:
    span = SpanInput()
    span.show()

col2.caption(
    '<div style="text-align: right">'
    f'Local time zone: <code>{datetime.now().astimezone().tzinfo}</code>'
    '</div>',
    unsafe_allow_html=True
)

st.divider()

TimePatternDistributionHistogram(
    props={
        'configs': configs_list.get_pattern_configs(),
        'colors': configs_list.get_pattern_colors(),
        'use_custom_span': not span.is_auto(),
        'span_expression': span.get_expression()
    }
).show()



import re
from datetime import timedelta


def validate_time_span(expression: str) -> bool:
    """Check whether the expression is valid representation of time
    span. The expression should have the following format:

    <expression> ::= <value><unit>
    <value> ::= <integer>
    <unit> ::= 'd' | 'h' | 'm' | 's'
    """
    return bool(re.match(r'^\d+[smhd]$', expression))


def parse_relative_time(expression: str) -> timedelta:
    """Parse `expression` representing relative time and return
    corresponding`timedelta` object. The expression format is next:

    <expression> ::= [<sign>]<term>{<term>}...
    <sign> ::= '+' | '-'
    <term> ::= <value><unit>
    <value> ::= <integer>
    <unit> ::= 'd' | 'h' | 'm' | 's'

    Example expressions: '+1d12h'; '1h30m10s'; '-3d4h'; '-1d2h30m';
    If expression cannot be properly parsed then `ValueError` is raised.
    """
    if not expression:
        raise ValueError('Empty expression is provided')

    pattern = (
        r'^(?P<sign>[-+])?'
        r'(?:(?P<days>\d+)d)?'
        r'(?:(?P<hours>\d+)h)?'
        r'(?:(?P<minutes>\d+)m)?'
        r'(?:(?P<seconds>\d+)s)?$'
    )
    match = re.match(pattern, expression)

    if match is None or match.start() != 0 or match.end() != (len(expression)):
        raise ValueError('Failed to parse expression')

    groups = match.groupdict()

    match groups.pop('sign'):
        case None:
            sign = 1
        case '+':
            sign = 1
        case '-':
            sign = -1
        case char:
            raise ValueError(f'Unexpected sign "{char}"')

    return timedelta(
        **{
            unit: int(value) * sign
            for unit, value in groups.items()
            if value is not None
        }
    )



from datetime import timedelta
from typing import Iterable

import numpy as np
import pandas as pd
import plotly.graph_objects as go  # type: ignore[import-untyped]
import streamlit as st
from eventum_plugins.input.base import InputPluginRuntimeError
from eventum_plugins.input.time_patterns import (TimePatternConfig,
                                                 TimePatternInputPlugin)
from numpy.typing import NDArray
from pytz import timezone
from tzlocal import get_localzone_name

from eventum_studio.components.component import BaseComponent
from eventum_studio.notifiers import NotificationLevel, default_notifier
from eventum_studio.utils.relative_time import parse_relative_time


def _hash_config(config: TimePatternConfig) -> int:
    return hash(
        (
            config.oscillator,
            config.multiplier,
            config.randomizer,
            config.spreader
        )
    )


@st.cache_data(
    max_entries=1024,
    show_spinner='Calculating distribution',
    hash_funcs={TimePatternConfig: _hash_config}
)
def _calculate_sample(config: TimePatternConfig) -> NDArray[np.datetime64]:
    """Calculate sample for specified `config`. If finite sample cannot
    be calculated then empty list is returned and corresponding
    notification is displayed."""
    pattern = TimePatternInputPlugin(
        config=config,
        tz=timezone(zone=get_localzone_name())
    )

    data = []
    pattern.sample(lambda ts: data.append(ts))

    return np.array(data)


class TimePatternDistributionHistogram(BaseComponent):
    """Component for visualizing time patterns distribution."""
    _AUTO_SPAN_BINS_COUNT = 60

    _SHOW_PROPS = {
        'configs': Iterable[TimePatternConfig],
        'colors': Iterable[str],
        'use_custom_span': bool,
        'span_expression': str
    }

    def _resample_series(self, series: pd.Series) -> pd.Series:
        """Resample series corresponding to specified span options. If
        auto span is used, then sample is resampled according to
        `_AUTO_SPAN_SAMPLE_SIZE`, otherwise `span_expression` prop is
        used.
        """
        use_custom_span: bool = self._props['use_custom_span']
        span_expression: str = self._props['span_expression']

        if use_custom_span:
            custom_span = parse_relative_time(span_expression)
            return series.resample(rule=custom_span).sum()
        else:
            auto_span = timedelta(
                seconds=(
                    (series.index[-1] - series.index[0])
                    / self._AUTO_SPAN_BINS_COUNT
                    / np.timedelta64(1000000, 'us')
                )
            )
            return series.resample(rule=auto_span).sum()

    def _show(self) -> None:
        configs: Iterable[TimePatternConfig] = self._props['configs']
        colors: Iterable[str] = self._props['colors']

        use_custom_span: bool = self._props['use_custom_span']
        span_expression: str = self._props['span_expression']

        traces: list[tuple[pd.Series, str, str]] = []
        total_events = 0

        min_timestamp = np.datetime64('9999-12-31')
        max_timestamp = np.datetime64('0000-01-01')

        for config, color in zip(configs, colors):
            try:
                series = pd.Series(1, index=_calculate_sample(config))
                total_events += series.size
            except InputPluginRuntimeError as e:
                default_notifier(
                    message=(
                        'Skip distribution calculation '
                        f'for pattern "{config.label}": {e}'
                    ),
                    level=NotificationLevel.WARNING
                )
                continue

            if not series.empty:
                series = self._resample_series(series)

                min_timestamp = min(min_timestamp, series.index[0])
                max_timestamp = max(max_timestamp, series.index[-1])

                traces.append((series, config.label, color))

        if use_custom_span:
            kwargs = {
                'xbins': {
                    'size': parse_relative_time(
                        span_expression
                    ).total_seconds() * 1000,
                    'start': min_timestamp,
                    'end': max_timestamp,
                }
            }
        else:
            kwargs = {'nbinsx': self._AUTO_SPAN_BINS_COUNT}     # type: ignore

        fig = go.Figure()

        for series, label, color in traces:
            fig.add_trace(
                go.Histogram(
                    x=series.index,
                    y=series.values,
                    histfunc='sum',
                    name=label,
                    marker_color=color,
                    **kwargs
                )
            )

        fig.update_layout(barmode='stack')
        st.plotly_chart(fig, use_container_width=True)

        col1, col2 = st.columns([8, 2])
        col1.text(f'Total events: {total_events}')
        col2.button(
            'Recalculate',
            use_container_width=True,
            key=self._wk.get_ephemeral(),
            on_click=_calculate_sample.clear,   # type: ignore[attr-defined]
            type='primary'
        )



from typing import Iterable

from pydantic_core import ErrorDetails


def prettify_errors(errors: list[ErrorDetails]) -> str:
    """Format pydantic dictionary gotten from `e.errors()` to pretty
    human readable string."""

    def _loc(location: Iterable[str | int]) -> str:
        """Format field location."""
        return '.'.join(map(str, location))

    messages: list[str] = []

    for error in errors:
        match error:
            case {'type': 'extra_forbidden', 'loc': loc}:
                msg = 'Field is unrecognized'
                messages.append(f'Field \"{_loc(loc)}\" - {msg}')
            case {'msg': msg, 'loc': loc}:
                messages.append(f'Field \"{_loc(loc)}\" - {msg}')

    return '; '.join(messages)



from typing import Callable

import streamlit as st

from eventum_studio.components.component import BaseComponent


class TemplateStateViewer(BaseComponent):
    """Component for displaying template state."""

    _SHOW_PROPS = {
        'local_vars': dict,
        'shared_vars': dict,
        'subprocess_commands_history': tuple[tuple[int, str]],
        'clear_state_callback': Callable[[], None]
    }

    def _show(self) -> None:
        st.caption(
            'State',
            help='Cross-rendering persisted state'
        )
        col1, col2, col3 = st.columns([1, 1, 1])
        with col1:
            st.caption(
                'Locals',
                help='Variables accessible within a template'
            )
            st.write(self._props['local_vars'])

        with col2:
            st.caption(
                'Shared',
                help='Variables shared between multiple templates'
            )
            st.write(self._props['shared_vars'])

        with col3:
            st.caption('Subprocess commands history')
            st.dataframe(
                data=self._props['subprocess_commands_history'],
                use_container_width=True,
                hide_index=True,
                column_config={
                    1: "#", 2: "Command"    # type: ignore[dict-item]
                }
            )

        _, col2 = st.columns([3, 1])
        col2.button(
            'Clear state',
            use_container_width=True,
            on_click=self._props['clear_state_callback']
        )



from typing import Callable, Optional, assert_never

import eventum_plugins.input.time_patterns as models
import streamlit as st
from eventum_content_manager.manage import (ContentManagementError,
                                            save_time_pattern)
from pydantic import ValidationError

from eventum_studio.components.component import BaseComponent
from eventum_studio.notifiers import NotificationLevel, default_notifier
from eventum_studio.utils.validation_prettier import prettify_errors


class TimePatternConfigurator(BaseComponent):
    """Component for controlling configuration of time pattern."""

    _STATE_INITIALIZATION_PROPS = {
        'initial_state': Optional[models.TimePatternConfig],
        'pattern_filename': Optional[str],
        'color': str
    }
    _SHOW_PROPS = {
        'save_callback': Callable[[str], None],
        'delete_callback': Callable
    }

    def _init_state(self) -> None:
        ss = self._session_state
        init = self._props['initial_state']

        if init is None:
            saved = False
            init = TimePatternConfigurator._get_default_configuration()
        else:
            saved = True

        ss['pattern_label'] = init.label
        ss['pattern_color'] = self._props['color']
        ss['is_saved'] = saved

        if saved:
            ss['pattern_filename'] = self._props['pattern_filename']

        ss['oscillator_period'] = init.oscillator.period
        ss['oscillator_period_unit'] = init.oscillator.unit
        ss['oscillator_start'] = str(init.oscillator.start)
        ss['oscillator_end'] = str(init.oscillator.end)

        ss['multiplier_ratio'] = init.multiplier.ratio

        ss['randomizer_deviation'] = init.randomizer.deviation
        ss['randomizer_direction'] = init.randomizer.direction

        ss['spreader_distribution'] = init.spreader.distribution
        self._set_distribution_parameters_in_state(init.spreader.parameters)

    def _set_default_distribution_parameters_in_state(self) -> None:
        ss = self._session_state

        ss['spreader_uniform_distr_bounds'] = (0.0, 1.0)

        ss['spreader_triangular_distr_last_changed'] = 'left'
        ss['spreader_triangular_distr_left_bounds'] = (0.0, 0.5)
        ss['spreader_triangular_distr_right_bounds'] = (0.5, 1.0)

        ss['spreader_beta_distr_a'] = 1.0
        ss['spreader_beta_distr_b'] = 1.0

    def _set_distribution_parameters_in_state(
        self,
        parameters: models.DistributionParameters
    ) -> None:
        self._set_default_distribution_parameters_in_state()

        ss = self._session_state
        match ss['spreader_distribution']:
            case models.Distribution.UNIFORM:
                ss['spreader_uniform_distr_bounds'] = (
                    parameters.low, parameters.high             # type: ignore
                )
            case models.Distribution.TRIANGULAR:
                ss['spreader_triangular_distr_left_bounds'] = (
                    parameters.left, parameters.mode            # type: ignore
                )
                ss['spreader_triangular_distr_right_bounds'] = (
                    parameters.mode, parameters.right           # type: ignore
                )
            case models.Distribution.BETA:
                ss['spreader_beta_distr_a'] = parameters.a      # type: ignore
                ss['spreader_beta_distr_b'] = parameters.b      # type: ignore
            case val:
                assert_never(val)

    def _get_current_distribution_parameters(
        self
    ) -> models.DistributionParameters:
        ss = self._session_state

        match ss['spreader_distribution']:
            case models.Distribution.UNIFORM:
                low, high = ss['spreader_uniform_distr_bounds']
                return models.UniformDistributionParameters(
                    low=low,
                    high=high
                )
            case models.Distribution.TRIANGULAR:
                left, mode = ss['spreader_triangular_distr_left_bounds']
                _, right = ss['spreader_triangular_distr_right_bounds']
                return models.TriangularDistributionParameters(
                    left=left,
                    mode=mode,
                    right=right
                )
            case models.Distribution.BETA:
                return models.BetaDistributionParameters(
                    a=ss['spreader_beta_distr_a'],
                    b=ss['spreader_beta_distr_b']
                )
            case val:
                assert_never(val)

    def _show_manage_section(self) -> None:
        st.text_input(
            'Label',
            key=self._wk('pattern_label'),
            help='Displayed name of time pattern'
        )
        st.text_input(
            'File name',
            key=self._wk('pattern_filename'),
            disabled=self._session_state['is_saved'],
            help='Path for current configuration'
        )

        if self._session_state['is_saved']:
            st.button(
                'Update',
                key=self._wk.get_ephemeral(),
                on_click=lambda: self._save(overwrite=True),
                use_container_width=True,
                type='primary'
            )
        else:
            st.button(
                'Save',
                key=self._wk.get_ephemeral(),
                on_click=lambda: self._save(),
                use_container_width=True,
                type='primary'
            )
        st.button(
            'Delete',
            key=self._wk.get_ephemeral(),
            on_click=lambda: self._props['delete_callback'](),
            use_container_width=True
        )

    def _show_oscillator_section(self) -> None:
        st.header(
            'Oscillator',
            help='When and how often to generate events'
        )
        col1, col2 = st.columns([3, 7])
        col1.number_input(
            'Period',
            step=1,
            min_value=1,
            key=self._wk('oscillator_period'),
            help='Period of time between events'
        )
        col2.selectbox(
            'Unit',
            options=[unit.value for unit in models.TimeUnit],
            key=self._wk('oscillator_period_unit'),
            help='Unit of time used in period'
        )
        col1, col2 = st.columns(2)
        col1.text_input(
            'Start time',
            key=self._wk('oscillator_start'),
            help=(
                'Start time of generating events. '
                'Can be set to time, datetime, relative time (e.g. `+1h30m`) '
                'or keyword `now`. '
                'Relative time is measured from current moment.'
            )
        )
        col2.text_input(
            'End time',
            key=self._wk('oscillator_end'),
            help=(
                'End time of generating events. '
                'Can be set to time, datetime, relative time (e.g. `+1h30m`) '
                'or keyword `never`. '
                'Relative time is measured from start time moment.'
            )
        )

    def _show_multiplier_section(self) -> None:
        st.header(
            'Multiplier',
            help='How many events to generate within the period'
        )
        st.number_input(
            'Ratio',
            step=1,
            min_value=1,
            key=self._wk('multiplier_ratio'),
            help='Event number multiplication ratio'
        )

    def _show_randomizer_section(self) -> None:
        st.header(
            'Randomizer',
            help='How much to randomize number of event within the period'
        )
        st.number_input(
            'Deviation',
            min_value=0.0,
            max_value=1.00,
            step=0.05,
            key=self._wk('randomizer_deviation'),
            help='Events number deviation ratio'
        )
        st.selectbox(
            'Direction',
            options=[
                direction.value
                for direction in models.RandomizerDirection
            ],
            key=self._wk('randomizer_direction'),
            help='Direction of number deviation'
        )

    def _show_spreader_parameters(self) -> None:
        ss = self._session_state
        match ss['spreader_distribution']:
            case models.Distribution.UNIFORM:
                st.slider(
                    'Bounds',
                    min_value=0.0,
                    max_value=1.0,
                    key=self._wk('spreader_uniform_distr_bounds')
                )
            case models.Distribution.TRIANGULAR:
                left, mode_l = ss['spreader_triangular_distr_left_bounds']
                mode_r, right = ss['spreader_triangular_distr_right_bounds']
                mode = (
                    mode_l
                    if ss['spreader_triangular_distr_last_changed'] == 'left'
                    else mode_r
                )

                ss['spreader_triangular_distr_left_bounds'] = (left, mode)
                ss['spreader_triangular_distr_right_bounds'] = (mode, right)

                st.slider(
                    'Increasing bounds',
                    min_value=0.0,
                    max_value=1.0,
                    key=self._wk('spreader_triangular_distr_left_bounds'),
                    on_change=(
                        lambda: ss.__setitem__(
                            'spreader_triangular_distr_last_changed',
                            'left'
                        )
                    )
                )
                st.slider(
                    'Decreasing bounds',
                    min_value=0.0,
                    max_value=1.0,
                    key=self._wk('spreader_triangular_distr_right_bounds'),
                    on_change=(
                        lambda: ss.__setitem__(
                            'spreader_triangular_distr_last_changed',
                            'right'
                        )
                    )
                )
            case models.Distribution.BETA:
                col1, col2 = st.columns([1, 1])
                col1.number_input(
                    'A',
                    min_value=0.0,
                    step=0.1,
                    key=self._wk('spreader_beta_distr_a')
                )
                col2.number_input(
                    'B',
                    min_value=0.0,
                    step=0.1,
                    key=self._wk('spreader_beta_distr_b')
                )
            case val:
                assert_never(val)

    def _show_spreader_section(self) -> None:
        st.header(
            'Spreader',
            help='How to spread events within the period'
        )
        st.selectbox(
            'Distribution',
            options=[
                func.value
                for func in models.Distribution
            ],
            key=self._wk('spreader_distribution'),
            help='Probability distribution function'
        )
        self._show_spreader_parameters()

    def _show(self):
        label = self._session_state['pattern_label']
        color = self._session_state['pattern_color']

        with st.expander(f':{color}[{label}]'):
            self._show_manage_section()
            st.divider()
            self._show_oscillator_section()
            st.divider()
            self._show_multiplier_section()
            st.divider()
            self._show_randomizer_section()
            st.divider()
            self._show_spreader_section()

    def _save(self, overwrite: bool = False) -> None:
        """Save currently configured time pattern to repository as
        configuration file.
        """
        try:
            save_time_pattern(
                config=self.get_current_configuration().model_dump(
                    mode='json'
                ),
                path=self._session_state['pattern_filename'],
                overwrite=overwrite
            )
        except ValidationError as e:
            default_notifier(
                message=(
                    'Failed to validate parameters in time pattern: '
                    f'{prettify_errors(e.errors())}'
                ),
                level=NotificationLevel.ERROR
            )
            return
        except ContentManagementError as e:
            default_notifier(f'Failed to save: {e}', NotificationLevel.ERROR)
            return

        self._props['save_callback'](self._session_state['pattern_filename'])

        self._session_state['is_saved'] = True
        default_notifier('Saved in repository', NotificationLevel.SUCCESS)

    def is_saved(self) -> bool:
        """Get status whether the time pattern is saved in repository."""
        return self._session_state['is_saved']

    def get_saved_filename(self) -> Optional[str]:
        """Get filename of time pattern if it is saved. If pattern is not
        saved then `None` will be returned.
        """
        if self._session_state['is_saved']:
            return self._session_state['pattern_filename']
        else:
            return None

    @property
    def label(self) -> str:
        """Get current label of time pattern."""
        return self._session_state['pattern_label']

    def get_current_configuration(self) -> models.TimePatternConfig:
        """Build TimePatternConfig from current input widgets values
        that are in the session state.
        """
        ss = self._session_state

        return models.TimePatternConfig(
            label=ss['pattern_label'],
            oscillator=models.OscillatorConfig(
                period=ss['oscillator_period'],
                unit=ss['oscillator_period_unit'],
                start=ss['oscillator_start'],
                end=ss['oscillator_end']
            ),
            multiplier=models.MultiplierConfig(
                ratio=ss['multiplier_ratio']
            ),
            randomizer=models.RandomizerConfig(
                deviation=ss['randomizer_deviation'],
                direction=ss['randomizer_direction']
            ),
            spreader=models.SpreaderConfig(
                distribution=ss['spreader_distribution'],
                parameters=self._get_current_distribution_parameters()
            )
        )

    @staticmethod
    def _get_default_configuration(
        label: str = 'New pattern'
    ) -> models.TimePatternConfig:
        """Create `TimePatternConfig` object with default values."""
        return models.TimePatternConfig(
            label=label,
            oscillator=models.OscillatorConfig(
                period=1,
                unit=models.TimeUnit.HOURS,
                start=models.TimeKeyword.NOW,
                end='+1h'
            ),
            multiplier=models.MultiplierConfig(ratio=1000),
            randomizer=models.RandomizerConfig(
                deviation=0,
                direction=models.RandomizerDirection.MIXED
            ),
            spreader=models.SpreaderConfig(
                distribution=models.Distribution.UNIFORM,
                parameters=models.UniformDistributionParameters(
                    low=0,
                    high=1
                )
            )
        )



from enum import Enum

import streamlit as st


class NotificationLevel(Enum):
    SUCCESS = 'success'
    INFO = 'info'
    WARNING = 'warning'
    ERROR = 'error'


class NotificationColor(Enum):
    SUCCESS = 'green'
    INFO = 'default'
    WARNING = 'orange'
    ERROR = 'red'


def null_notifier(
    message: str,
    level: NotificationLevel = NotificationLevel.INFO,
) -> None:
    pass


def streamlit_toast_notifier(
    message: str,
    level: NotificationLevel = NotificationLevel.INFO,
) -> None:
    match level:
        case NotificationLevel.SUCCESS:
            fmt_msg = f':{NotificationColor.SUCCESS.value}[{message}]'
        case NotificationLevel.WARNING:
            fmt_msg = f':{NotificationColor.WARNING.value}[{message}]'
        case NotificationLevel.ERROR:
            fmt_msg = f':{NotificationColor.ERROR.value}[{message}]'
        case _:
            fmt_msg = message

    st.toast(fmt_msg)


default_notifier = streamlit_toast_notifier



from datetime import timedelta

import pytest

from eventum_studio.utils.relative_time import (parse_relative_time,
                                                validate_time_span)


def test_parse_relative_time():
    assert parse_relative_time('1d') == timedelta(days=1)
    assert parse_relative_time('12h') == timedelta(hours=12)
    assert parse_relative_time('15m') == timedelta(minutes=15)
    assert parse_relative_time('30s') == timedelta(seconds=30)
    assert parse_relative_time('-1d') == timedelta(days=-1)
    assert parse_relative_time('-15m2s') == timedelta(minutes=-15, seconds=-2)
    assert parse_relative_time('-1d30s') == timedelta(days=-1, seconds=-30)
    assert parse_relative_time('-12h10m15s') == timedelta(
        hours=-12, minutes=-10, seconds=-15
    )
    assert parse_relative_time('50d60h70m80s') == timedelta(
        days=50, hours=60, minutes=70, seconds=80
    )
    assert parse_relative_time('-50d60h70m80s') == timedelta(
        days=-50, hours=-60, minutes=-70, seconds=-80
    )
    assert parse_relative_time('+15d15h30m24s') == timedelta(
        days=15, hours=15, minutes=30, seconds=24
    )

    with pytest.raises(ValueError):
        parse_relative_time('16Days minus three hours')

    with pytest.raises(ValueError):
        parse_relative_time('16d3j')

    with pytest.raises(ValueError):
        parse_relative_time('')


def test_validate_time_span():
    assert validate_time_span('0s') is True
    assert validate_time_span('1s') is True
    assert validate_time_span('15m') is True
    assert validate_time_span('12h') is True
    assert validate_time_span('7d') is True
    assert validate_time_span('120m') is True

    assert validate_time_span('12') is False
    assert validate_time_span('1M') is False
    assert validate_time_span('6hour') is False
    assert validate_time_span('-2m') is False
    assert validate_time_span('14mh') is False
    assert validate_time_span('s') is False



from typing import Callable

import streamlit as st
from streamlit_elements import editor  # type: ignore[import-untyped]
from streamlit_elements import elements, event, lazy

from eventum_studio.components.component import BaseComponent
from eventum_studio.components.sample_explorer import SampleExplorer
from eventum_studio.notifiers import NotificationLevel, default_notifier


class TemplateConfigurationEditor(BaseComponent):
    """Component for editing configuration for template."""

    _SHOW_PROPS = {
        'on_change': Callable[[str], None]
    }

    _DEFAULT_CONTENT = (
        'params: { }\n'
        'samples: { }\n'
    )

    def _init_state(self) -> None:
        self._session_state['content'] = self._DEFAULT_CONTENT

    def _handle_on_change(self, value: str) -> None:
        """Handle changes committing."""
        self._session_state['content'] = value
        self._props['on_change'](value)

    def _show(self) -> None:
        col1, col2 = st.columns([1, 1])

        with col1:
            st.caption(
                'Configuration',
                help=(
                    'Edit the configuration for template rendering '
                    'and use `Ctrl+S` to commit changes'
                )
            )

            with elements(self._wk('configuration_editor')):
                event.Hotkey(
                    sequence='ctrl+s',
                    callback=(
                        lambda:
                        default_notifier(
                            message='Configuration is updated',
                            level=NotificationLevel.INFO
                        )
                    ),
                    bindInputs=True,
                    overrideDefault=True
                )

                editor.Monaco(
                    theme='vs-dark',
                    language='yaml',
                    height=670,
                    value=self._DEFAULT_CONTENT,
                    onChange=lazy(self._handle_on_change),
                    options={
                        'cursorSmoothCaretAnimation': True,
                        'tabSize': 2
                    }
                )

        with col2:
            SampleExplorer(
                widget_keys_context=self._wk,
                props={'display_size': 15}
            ).show()

    @property
    def content(self) -> str:
        """Get currently committed content in editor."""
        return self._session_state['content']
